package com.bocloud.sms.booter.config;

import com.bocloud.cmp.boot.model.BoCloudExchange;
import com.bocloud.cmp.core.encrypt.AbstractEncryptHandler;
import com.bocloud.sms.booter.listener.BocloudMessageListener;
import com.megatron.common.utils.IDFactory;
import com.megatron.common.utils.MapTools;
import com.megatron.framework.core.CurrentService;
import org.springframework.amqp.core.*;
import org.springframework.amqp.rabbit.connection.ConnectionFactory;
import org.springframework.amqp.rabbit.core.RabbitAdmin;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.amqp.rabbit.listener.SimpleMessageListenerContainer;
import org.springframework.amqp.support.converter.Jackson2JsonMessageConverter;
import org.springframework.amqp.support.converter.MessageConverter;
import org.springframework.beans.factory.config.ConfigurableBeanFactory;
import org.springframework.boot.autoconfigure.condition.ConditionalOnBean;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Scope;

import java.util.Map;

/**
 * rabbitmq配置类
 *
 * @author wangyu
 * @version 1.0
 * @since 2018年3月30日
 */
@Configuration
public class RabbitMQConfiguration {

    private final ConnectionFactory connectionFactory;
    private final Map<String, Object> arguments = MapTools.simpleMap("x-ha-policy", "all");

    public RabbitMQConfiguration(ConnectionFactory connectionFactory) {
        this.connectionFactory = connectionFactory;
    }

    /**
     * 声明主题交换机
     *
     * @return 交换机
     */
    @Bean
    public TopicExchange topicExchange() {
        return new TopicExchange(BoCloudExchange.Default.TOPIC.getExchange(), true, true);
    }

    /**
     * 声明默认交换机
     *
     * @return 交换机
     */
    @Bean
    public DirectExchange defaultExchange() {
        return new DirectExchange(BoCloudExchange.Default.DIRECT.getExchange(), true, true);
    }

    @Bean
    public Binding bindingMessageQueue(CurrentService service) {
        return BindingBuilder.bind(declareMessageQueue(service)).to(topicExchange()).with("bocloud.message.queue.#");
    }

    @Bean
    public MessageConverter messageConverter() {
        return new Jackson2JsonMessageConverter();
    }

    @Bean
    public Queue declareMessageQueue(CurrentService service) {
        return new Queue("bocloud.message.queue.sms." + service.getToken(), true, false, true, arguments);
    }

    /**
     * 声明审计日志队列
     *
     * @return 队列
     */
    @Bean
    public Queue declareLogQueue() {
        return new Queue("bocloud.log.queue", true, false, false, arguments);
    }

    /**
     * 声明通知消息队列
     *
     * @return 队列
     */
    @Bean
    public Queue declareNoticeQueue() {
        return new Queue("event.notice.queue", true, false, false, arguments);
    }

    @Bean
    public Binding bindingNoticeQueue() {
        return BindingBuilder.bind(declareNoticeQueue()).to(defaultExchange()).withQueueName();
    }

    @Bean
    public Binding bindingLogQueue() {
        return BindingBuilder.bind(declareLogQueue()).to(defaultExchange()).withQueueName();
    }

    @Bean
    @ConditionalOnBean(AbstractEncryptHandler.class)
    public Queue declareLicenseInstallQueue() {
        return new Queue("license.install.queue." + IDFactory.instance().uuid(), true, false, true, arguments);
    }

    @Bean
    @ConditionalOnBean(AbstractEncryptHandler.class)
    public Binding bindLicenseInstallQueue() {
        return BindingBuilder.bind(declareLicenseInstallQueue()).to(defaultExchange()).with("license.install.queue");
    }

    @Bean
    @Scope(ConfigurableBeanFactory.SCOPE_PROTOTYPE)
    public RabbitTemplate amqpTemplate() {
        RabbitTemplate amqpTemplate = new RabbitTemplate(connectionFactory);
        amqpTemplate.setMessageConverter(messageConverter());
        return amqpTemplate;
    }

    @Bean
    public RabbitAdmin rabbitAdmin() {
        return new RabbitAdmin(amqpTemplate());
    }

    @Bean
    public SimpleMessageListenerContainer autoMessageListener(CurrentService service, BocloudMessageListener bocloudMessageListener) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        container.setQueues(declareMessageQueue(service));
        // 设置确认模式自动确认
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(bocloudMessageListener);
        container.setAmqpAdmin(rabbitAdmin());
        container.setAutoStartup(true);
        return container;
    }

    @Bean
    @ConditionalOnBean(AbstractEncryptHandler.class)
    public SimpleMessageListenerContainer licenseInstallListener(AbstractEncryptHandler encryptHandler) {
        SimpleMessageListenerContainer container = new SimpleMessageListenerContainer(connectionFactory);
        // 设置确认模式自动确认
        container.setQueues(declareLicenseInstallQueue());
        container.setAcknowledgeMode(AcknowledgeMode.AUTO);
        container.setMessageListener(encryptHandler);
        container.setAmqpAdmin(rabbitAdmin());
        container.setAutoStartup(true);
        return container;
    }
}
