package com.bocloud.sms.booter.controller;

import com.alibaba.fastjson.JSONArray;
import com.bocloud.sms.entity.*;
import com.bocloud.sms.interfaces.TenantService;
import com.bocloud.sms.model.*;
import com.megatron.common.model.*;
import com.megatron.common.utils.Common;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.Assert;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import java.util.List;
import java.util.Map;

/**
 * @author admin
 */
@RestController
@RequestMapping("/v1/tenants")
@Tag(name = "租户管理")
@Slf4j
public class TenantController {


    private final TenantService tenantService;

    public TenantController(TenantService tenantService) {
        this.tenantService = tenantService;
    }

    /**
     * 添加租户
     *
     * @param tenant
     * @param requestContext
     * @return
     */
    @Operation(summary = "创建租户")
    @PostMapping
    public Result create(@RequestBody TenantModel tenant,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.create(tenant, requestContext);
    }

    /**
     * 修改租户
     *
     * @param id
     * @param tenant
     * @param requestContext
     * @return
     */
    @Operation(summary = "修改租户")
    @PutMapping("/{id}")
    public Result modify(@PathVariable(value = Common.ID) Long id,
                         @RequestBody TenantModel tenant,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.modify(id, tenant, requestContext);
    }

    /**
     * 移除租户
     *
     * @param id
     * @param requestContext
     * @return
     */
    @Operation(summary = "删除租户")
    @DeleteMapping("/{id}")
    public Result remove(@PathVariable(value = Common.ID) Long id,
                         @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.remove(id, requestContext);
    }

    /**
     * 查看租户详情
     *
     * @param id
     * @return
     */
    @Operation(summary = "查询租户详情")
    @GetMapping("/{id}")
    public GeneralResult<Tenant> detail(@PathVariable(value = Common.ID) Long id) {
        return tenantService.detail(id);
    }

    /**
     * 锁定租户
     *
     * @param account 租户账号
     * @return 锁定结果
     */
    @Operation(summary = "锁定租户")
    @PutMapping()
    public Result tenantLock(@RequestParam(value = Common.ACCOUNT) String account) {
        return tenantService.lockByAccount(account);
    }

    /**
     * 租户根据账号获取登录需要信息
     *
     * @param account 账号
     * @return 登录信息
     */
    @Operation(summary = "获取租户信息")
    @PostMapping("/info")
    public GeneralResult<Map<String, Object>> tenantAccountInfo(@RequestParam(value = Common.ACCOUNT) String account) {
        return tenantService.tokenLogin(account);
    }


    /**
     * 获取某个租户下的所有项目
     *
     * @param id
     * @return
     */
    @Operation(summary = "获取项目列表")
    @GetMapping("/{id}/projects")
    public GeneralResult<List<Project>> projects(@PathVariable(value = Common.ID) Long id) {
        return tenantService.projects(id);
    }


    /**
     * 分页查询租户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Operation(summary = "分页获取租户列表")
    @GetMapping
    public GeneralResult<GridBean<Tenant>> list(Pager pager, @Value(value = Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.list(pager, requestContext);
    }

    /**
     * 修改租户状态
     *
     * @param id
     * @param action
     * @param requestContext
     * @return
     */
    @Operation(summary = "租户操作")
    @PatchMapping("/{id}")
    public Result updateStatus(@PathVariable(value = Common.ID) Long id,
                               @RequestBody ActionModel action,
                               @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.updateStatus(id, action.getAction(), requestContext);
    }

    /**
     * 修改租户密码
     *
     * @param id             租户id
     * @param passwordModel  密码对象
     * @param requestContext 请求对象
     * @return
     */
    @Operation(summary = "修改租户密码")
    @PatchMapping("/{id}/pwd")
    public Result changePwd(@PathVariable(value = Common.ID) Long id,
                            @RequestBody PasswordModel passwordModel,
                            @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.changePwd(id, passwordModel.getOldPassword(), passwordModel.getPassword(), requestContext);
    }

    /**
     * 重置租户密码
     *
     * @param id             租户id
     * @param passwordModel  密码对象
     * @param requestContext 请求对象
     * @return
     */
    @Operation(summary = "重置租户密码")
    @PatchMapping("/{id}/reset")
    public Result resetPwd(@PathVariable(value = Common.ID) Long id,
                           @RequestBody PasswordModel passwordModel,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.resetPwd(id, passwordModel.getPassword(), requestContext);
    }


    /**
     * 检查租户是否异地登录
     *
     * @param id
     * @param request
     * @return
     */
    @Operation(summary = "检查租户是否登录")
    @GetMapping("/{id}/status")
    public Result checkTenant(@PathVariable(value = Common.ID) Long id,
                              HttpServletRequest request) {
        return tenantService.checkTenant(id, request.getHeader(Common.TOKEN));
    }

    /**
     * 租户的角色列表
     *
     * @param id
     * @return
     */
    @Operation(summary = "查询租户的角色列表")
    @GetMapping("/{id}/roles")
    public GeneralResult<List<Role>> roles(@PathVariable(value = Common.ID) Long id) {
        return tenantService.listRoles(id);
    }

    /**
     * 获取租户安全对象
     *
     * @param apiKey
     * @return
     */
    @Operation(summary = "获取租户安全对象")
    @GetMapping("/security")
    public GeneralResult<AccountSecurity> security(@RequestParam(value = "apiKey") String apiKey) {
        return tenantService.getByApiKey(apiKey);
    }

    /**
     * 批量操作租户
     *
     * @param batchAction
     * @return
     */
    @Operation(summary = "批量操作租户")
    @PatchMapping
    public Result batchLock(@RequestBody BatchActionModel batchAction, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        switch (batchAction.getAction()) {
            case lock:
                return tenantService.batchLock(batchAction.getIds(), requestContext);
            case active:
                return tenantService.batchActive(batchAction.getIds(), requestContext);
            case remove:
                return tenantService.batchRemove(batchAction.getIds(), requestContext);
            default:
                return new Result(false, "不支持这种操作");
        }
    }

    /**
     * 删除租户资源池关系
     *
     * @param poolId
     * @return
     */
    @Operation(summary = "删除租户资源池关联")
    @DeleteMapping("/pools")
    public Result poolDelete(@RequestParam(value = "poolId") Long poolId) {
        return tenantService.deleteByPoolId(poolId);
    }


    /**
     * 根据租户查询资源池关联信息
     *
     * @param id
     * @return
     */
    @Operation(summary = "查询租户资源池关联")
    @GetMapping("/{id}/pools")
    public GeneralResult<List<TenantPool>> businessPools(@PathVariable(value = Common.ID) Long id) {
        return tenantService.listTenantPool(id);
    }

    /**
     * 租户配置资源池
     *
     * @param id
     * @param poolIds
     * @return
     */
    @Operation(summary = "租户配置资源池")
    @PostMapping("/{id}/pools")
    public Result configPool(@PathVariable(value = Common.ID) Long id, @RequestBody List<Long> poolIds, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.configPool(id, null, poolIds, requestContext);
    }

    /**
     * 租户配置业务
     *
     * @param id
     * @param businessIds
     * @return
     */
    @Operation(summary = "租户配置业务")
    @PostMapping("/{id}/businesses")
    public Result configBusiness(@PathVariable(value = Common.ID) Long id, @RequestParam(value = "businessIds") String businessIds) {
        List<Long> businesses = JSONArray.parseArray(businessIds, Long.class);
        return tenantService.configBusiness(id, businesses);
    }

    /**
     * 登录记录查询
     *
     * @param pager
     * @return
     */
    @Operation(summary = "登录记录查询")
    @GetMapping("/track")
    public GeneralResult<GridBean<LoginRecord>> records(Pager pager) {
        return this.tenantService.list(pager);
    }

    @Operation(summary = "添加收藏")
    @PostMapping("/favorites/{id}")
    public Result addFavorite(@PathVariable("id") Long serviceId, @RequestParam("tenantId") Long tenantId) {
        return tenantService.addFavorite(serviceId, tenantId);
    }

    @Operation(summary = "批量添加收藏")
    @PostMapping("/favorites")
    public Result addFavorites(@RequestBody List<Favorite> favorites) {
        return tenantService.addFavorites(favorites);
    }

    @Operation(summary = "删除收藏")
    @DeleteMapping("/favorites")
    public Result cancelTenantFavorite(@RequestParam(value = "serviceId", required = false) Long serviceId, @RequestParam(value = "tenantId", required = false) Long tenantId, @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.cancelFavorite(serviceId, tenantId, requestContext);
    }

    @Operation(summary = "获取收藏夹")
    @GetMapping("/favorites")
    public GeneralResult<List<CloudServiceBean>> listFavorite(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.listFavorite(requestContext);
    }

    @Operation(summary = "获取服务目录")
    @GetMapping("/services")
    public GeneralResult<List<ServiceMenuBean>> listServiceMenu(@Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.listServiceMenu(requestContext);
    }

    @Operation(summary = "修改收藏夹排序")
    @PostMapping("/favorites/order")
    public Result changePriority(@RequestBody List<PriorityModel> priorityModel,
                                 @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        return tenantService.changePriority(priorityModel, requestContext);
    }

    @Operation(summary = "查询租户下用户")
    @GetMapping("/{id}/users")
    public GeneralResult<List<User>> getTenantUsers(@PathVariable(value = Common.ID) Long id) {
        return tenantService.getTenantUsers(id);
    }

    @Operation(summary = "租户配置用户")
    @PostMapping("/{id}/users")
    public Result configUsers(@PathVariable(value = Common.ID) Long id, @RequestBody List<Long> userIds) {
        return tenantService.configUsers(id, userIds);
    }

    @Operation(summary = "查询租户账号是否存在")
    @GetMapping("/{account}/exist")
    public Result exist(@PathVariable(value = Common.ACCOUNT) String account) {
        return tenantService.exist(account);
    }

    @Operation(summary = "根据省市编码查询租户")
    @GetMapping("/{city}/list")
    public GeneralResult<List<Tenant>> listTenantByCity(@PathVariable(value = "city") String city) {
        return tenantService.listTenantByCity(city);
    }

    /**
     * 导出租户列表
     *
     * @param pager
     * @param requestContext
     * @return
     */
    @Deprecated
    @Operation(summary = "导出租户列表")
    @RequestMapping(value = "/export", method = {RequestMethod.GET})
    public void export(Pager pager, @Value(Common.REQ_CONTEXT) RequestContext requestContext,
                       HttpServletRequest request, HttpServletResponse response) {
        List<Param> paramList = pager.getParams();
        Result result = tenantService.export(paramList, requestContext, response);
        Assert.isTrue(result.isSuccess(), "导出租户列表异常");
    }

    @Operation(summary = "下载租户列表模板")
    @RequestMapping(value = "/import/template", method = {RequestMethod.GET})
    public void importTemp(HttpServletRequest request, HttpServletResponse response,
                           @Value(Common.REQ_CONTEXT) RequestContext requestContext) {
        Result result = tenantService.importTemp("租户列表模板", requestContext, response);
        Assert.isTrue(result.isSuccess(), result.getMessage());
    }

    @Operation(summary = "导入租户列表")
    @PostMapping(value = "/import")
    public Result importData(@Value(Common.REQ_CONTEXT) RequestContext requestContext, HttpServletRequest request) {
        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        List<MultipartFile> fileList = multipartRequest.getFiles("file");
        if (fileList.isEmpty()) {
            return new Result(false, "文件错误");
        }
        MultipartFile multipartFile = fileList.get(0);
        String originalFilename = multipartFile.getOriginalFilename();
        // 判断文件名
        if (originalFilename == null || !(originalFilename.endsWith(".xls") || originalFilename.endsWith(".xlsx"))) {
            return new Result(false, "文件错误");
        }
        return tenantService.importData(multipartFile, requestContext);
    }

    @Operation(summary = "查询租户账号是否存在")
    @GetMapping("/condition")
    public GeneralResult condition(@RequestParam(value = Common.CONDITION, required = false) String condition) {
        return tenantService.condition(condition);
    }
}
