package com.bocloud.sms.booter.listener;

import com.alibaba.fastjson.JSONObject;
import com.bocloud.sms.entity.Tenant;
import com.bocloud.sms.entity.User;
import com.bocloud.sms.interfaces.MessageService;
import com.bocloud.sms.model.NotifyMessage;
import com.bocloud.sms.repository.TenantRepository;
import com.bocloud.sms.repository.UserRepository;
import com.bocloud.sms.service.utils.DingTalkSender;
import com.bocloud.sms.service.utils.EmailSender;
import com.bocloud.sms.service.utils.WeChatWorkSender;
import com.megatron.common.model.RequestContext;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * 事件通知监听器
 *
 * @author weiwei
 * @since 2017年7月26日
 */
@RequiredArgsConstructor
@Component("eventListener")
@Slf4j
public class EventListener {

    private final MessageService messageService;
    private final EmailSender emailSender;
    private final UserRepository userRepository;
    private final TenantRepository tenantRepository;
    private final DingTalkSender dingTalkSender;
    private final WeChatWorkSender weChatWorkSender;

    @RabbitListener(queues = "event.notice.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void onMessage(NotifyMessage message) {
        log.debug("接收到时间通知消息：{}", JSONObject.toJSON(message));
        try {
            // 主题
            String subject = message.getSubject();
            // 站内信内容
            String msgContent = message.getContent();
            // 邮件内容
            String emailContent = StringUtils.hasText(message.getEmailContent()) ? message.getEmailContent() : message.getContent();
            // 企业微信内容
            String weChatContent = StringUtils.hasText(message.getWeChatContent()) ? message.getWeChatContent() : message.getContent();
            // 钉钉内容
            String dingTalkContent = StringUtils.hasText(message.getDingTalkContent()) ? message.getDingTalkContent() : message.getContent();
            RequestContext context = message.getContext();
            List<RequestContext> contexts = Optional.ofNullable(message.getContexts()).orElse(new ArrayList<>());
            if (context != null) {
                contexts.add(context);
            }
            //收件邮箱
            String[] emails = null;
            //接收者钉钉唯一标识
            String dingTalkUserId = null;
            //企业微信唯一标识
            String weChatUserId = null;
            for (RequestContext requestContext : contexts) {
                if (RequestContext.Catalog.Tenant == requestContext.getCatalog()) {
                    Tenant tenant = tenantRepository.query(requestContext.getTarget());
                    if (tenant != null) {
                        if (StringUtils.hasText(tenant.getEmail())) {
                            emails = new String[]{tenant.getEmail()};
                        }
                        if (StringUtils.hasText(tenant.getDingTalkNumber())) {
                            dingTalkUserId = tenant.getDingTalkNumber();
                        }
                        if (StringUtils.hasText(tenant.getWeChatWorkNumber())) {
                            weChatUserId = tenant.getWeChatWorkNumber();
                        }
                    }
                } else {
                    User user = userRepository.query(requestContext.getTarget());
                    if (user != null) {
                        if (StringUtils.hasText(user.getEmail())) {
                            emails = new String[]{user.getEmail()};
                        }
                        if (StringUtils.hasText(user.getDingtalk())) {
                            dingTalkUserId = user.getDingtalk();
                        }
                        if (StringUtils.hasText(user.getWechat())) {
                            weChatUserId = user.getWechat();
                        }
                    }
                }
                // 发送站内信
                if (message.isSendMsg()) {
                    messageService.create(requestContext, subject, msgContent);
                }
                // 发送邮件
                if (message.isSendEmail() && null != emails) {
                    if (emailContent == null) {
                        emailSender.send(subject, emails, msgContent, null);
                    } else {
                        emailSender.send(subject, emails, emailContent, null);
                    }
                }
                if (message.isSendDingTalk() && StringUtils.hasText(dingTalkUserId)) {
                    //发送钉钉消息
                    dingTalkSender.sendMessage(dingTalkContent, dingTalkUserId, null, false);
                }
                if (message.isSendWeChat() && StringUtils.hasText(weChatUserId)) {
                    //发送企业微信消息
                    weChatWorkSender.pushMessage(weChatUserId, weChatContent);
                }
            }
        } catch (Exception e) {
            log.error("Send message or email error:", e);
        }
    }
}
