package com.bocloud.sms.booter.listener;

import com.bocloud.sms.entity.AccessLog;
import com.bocloud.sms.interfaces.LogService;
import com.megatron.common.utils.IDFactory;
import jakarta.annotation.PreDestroy;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.annotation.RabbitListener;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

/**
 * 审计日志监听器，用来监听从rabbitmq日志队列发出的日志消息
 *
 * @author dmw
 */
@Component("logListener")
@Slf4j
public class LogListener implements InitializingBean {

    private BlockingQueue<AccessLog> accessLogQueue = new LinkedBlockingQueue<>();
    private volatile boolean runnable = true;
    @Autowired
    private LogService logService;

    /**
     * 处理访问日志消息，用来将审计日志存储到数据库
     *
     * @param log
     */
    @RabbitListener(queues = "bocloud.log.queue", admin = "rabbitAdmin", autoStartup = "true")
    public void processAccessLogMessage(AccessLog log) {
        log.setId(IDFactory.instance().uuid());
        if ("系统配置".equals(log.getTarget()) && "更新操作".equals(log.getAction())) {
            if (log.getDetail().length() > 65535) {
                log.setDetail(log.getDetail().substring(0, 65534).replace("/", ""));
            }

        }
        accessLogQueue.add(log);
    }

    class BatchSaver implements Runnable {
        long start = System.currentTimeMillis();

        /**
         * 获取日志队列
         *
         * @return
         */
        private List<AccessLog> fetchLog() {
            List<AccessLog> logList = new ArrayList<>();
            try {
                while ((System.currentTimeMillis() - start) < 1000 && logList.size() < 512) {
                    logList.add(accessLogQueue.take());
                }
            } catch (Exception e) {
                log.error("Fetch log Error！", e);
            }
            return logList;
        }

        @Override
        public void run() {
            while (runnable) {
                List<AccessLog> logs = fetchLog();
                if (logs.isEmpty()) {
                    continue;
                }
                logService.batch(logs);
                start = System.currentTimeMillis();
            }
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        new Thread(new BatchSaver()).start();
    }

    @PreDestroy
    public void destory() {
        this.runnable = false;
    }
}
