package com.bocloud.sms.repository;

import com.bocloud.sms.entity.User;
import com.megatron.common.enums.BaseStatus;
import com.megatron.common.model.Pager;
import com.megatron.common.model.Param;
import com.megatron.common.model.SimpleBean;
import com.megatron.common.utils.Common;
import com.megatron.common.utils.MapTools;
import com.megatron.database.core.intf.impl.JdbcGenericDao;
import com.megatron.database.utils.QueryBuilder;
import com.megatron.framework.core.CurrentService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.namedparam.NamedParameterJdbcTemplate;
import org.springframework.stereotype.Repository;
import org.springframework.util.Assert;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tyl
 * @since 2020/2/3 周一 11:37
 */
@Repository
public class UserRepository extends JdbcGenericDao<User, Long> {
    @Autowired
    public UserRepository(JdbcTemplate jdbcTemplate, NamedParameterJdbcTemplate npJdbcTemplate, CurrentService service) {
        super(jdbcTemplate, npJdbcTemplate, service);
    }

    /**
     * 删除用户
     *
     * @param id
     * @param menderId
     * @return
     */
    public boolean delete(Long id, Long menderId) {
        String sql = "update users set is_deleted = 1, gmt_modify = :gmtModify, mender_id = :menderId where id = :id";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        return this.execute(sql, params) > 0;
    }

    /**
     * 锁定用户
     *
     * @param id
     * @param menderId
     * @return
     */
    public boolean lock(Long id, Long menderId) {
        return updateStatus(id, menderId, BaseStatus.ABNORMAL);
    }

    /**
     * 批量冻结用户
     *
     * @param users
     * @param menderId
     * @return
     */
    public boolean batchLock(List<User> users, Long menderId) {
        for (User user : users) {
            user.setStatus(BaseStatus.ABNORMAL.name());
            user.setMenderId(menderId);
        }
        return batchUpdate(users, users.size());
    }

    public boolean batchUpdateStatus(List<Long> ids, Long menderId, BaseStatus status) {
        String sql =
            "update users set status = :status, gmt_modify = :gmtModify, mender_id = :menderId  where id in (:ids) and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("ids", ids);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        params.put("status", status.name());
        return this.execute(sql, params) > 0;
    }

    /**
     * 批量解冻用户
     *
     * @param users
     * @param menderId
     * @return
     */
    public boolean batchActive(List<User> users, Long menderId) {
        for (User user : users) {
            user.setStatus(BaseStatus.NORMAL.name());
            user.setMenderId(menderId);
        }
        return batchUpdate(users, users.size());
    }

    /**
     * 激活用户
     *
     * @param id
     * @param menderId
     * @return
     */
    public boolean active(Long id, Long menderId) {
        return updateStatus(id, menderId, BaseStatus.NORMAL);
    }

    private boolean updateStatus(Long id, Long menderId, BaseStatus status) {
        String sql =
            "update users set status = :status, gmt_modify = :gmtModify, mender_id = :menderId  where id = :id and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap("id", id);
        params.put("gmtModify", new Date());
        params.put("menderId", menderId);
        params.put("status", status.name());
        return this.execute(sql, params) > 0;
    }

    /**
     * 根据账号查询用户
     *
     * @param account
     * @return
     */
    public User getByAccount(String account) {
        String sql =
            "select u.*, t.account tenant_account,t.name tenant_name, t.discount discount from users u left join tenant t on u.tenant_id = t.id where u.is_deleted = 0 and u.account = :account ";
        Map<String, Object> params = MapTools.simpleMap("account", account);
        List<User> list = this.list(User.class, sql, params);
        Assert.isTrue(list.size() <= 1, "该账号已存在");
        return list.isEmpty() ? null : list.get(0);
    }

    public User getByUserId(String userId) {
        String sql =
                "select u.*, t.account tenant_account,t.name tenant_name, t.discount discount from users u left join tenant t on u.tenant_id = t.id where u.is_deleted = 0 and u.user_id = :userId ";
        Map<String, Object> params = MapTools.simpleMap("userId", userId);
        List<User> list = this.list(User.class, sql, params);
//        Assert.isTrue(list.size() <= 1, "该天融信用户已存在");
        return list.isEmpty() ? null : list.get(0);
    }
    /**
     * 分页查询用户
     *
     * @param page
     * @param rows
     * @param params
     * @param sorter
     * @return @
     */
    public List<User> list(int page, int rows, List<Param> params, Map<String, String> sorter, Long roleId) {
        String sql;
        if (null != roleId && 0L != roleId) {
            sql = "select a.*,d.name depart_name from users a left join manager_role b on a.id = b.manager_id"
                + " left join department d on d.id = a.depart_id where a.is_deleted = 0 and b.role_id = " + roleId;
        } else {
            sql =
                "select a.*,b.name tenant_name,b.id tenant_id from users a left join tenant b on b.id = a.tenant_id where a.is_deleted = 0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        // 20220319 这里如果roleId是null会导致bre报错，而且这行毛细用不到吧，直接干掉
        // paramMap.put("roleId", roleId);
        return this.list(User.class, sql, paramMap);
    }

    /**
     * 简易查询用户
     *
     * @param params
     * @param sorter
     * @return @
     */
    public List<SimpleBean> list(List<Param> params, Map<String, String> sorter, Long roleId) {
        String sql;
        if (null != roleId && 0L != roleId) {
            sql =
                "select  a.* from users a left join manager_role b on a.id = b.manager_id  where  a.is_deleted = 0 and b.role_id = "
                    + roleId;
        } else {
            sql = "select a.* from users a where a.is_deleted = 0 ";
        }
        sql = getQueryBuilder().buildRaw(sql, new Pager(1, Integer.MAX_VALUE, params, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(User.class, sql, paramMap).stream()
            .map(object -> new SimpleBean(object.getId(), object.getName(), object.getAccount()))
            .collect(Collectors.toList());
    }

    public List<User> list() {
        String sql = "select a.* from users a where a.is_deleted = 0 ";
        return this.list(User.class, sql);
    }


    public List<User> listAll() {
        String sql = "select a.* from users a";
        return this.list(User.class, sql);
    }

    /**
     * 查询用户数量
     *
     * @param params
     * @return
     */
    public int count(List<Param> params, Long roleId) {
        String sql;
        if (null == roleId || 0L == roleId) {
            sql = "select count(a.id) from users a where a.is_deleted = 0 ";
        } else {
            sql =
                "select count(a.id) from users a left join manager_role b on a.id = b.manager_id where a.is_deleted = 0 and b.role_id = "
                    + roleId;
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        paramMap.put("roleId", roleId);
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 查询用户数量
     *
     * @param params
     *            查询参数
     * @param projectIds
     *            项目id
     * @param roleId
     *            角色id
     * @return
     */
    public int count(List<Param> params, List<Long> projectIds, Long roleId) {
        String sql;
        if (null != roleId) {
            if (projectIds.size() > 0) {
                sql =
                    "select count(distinct a.id) from users a left join user_role b on a.id = b.user_id left join user_project c on a.id = c.user_id where a.is_deleted = 0 "
                        + " and b.role_id = " + roleId + "and c.project_id in (:projectIds) ";
            } else {
                sql =
                    "select count(1) from users a left join user_role b on a.id = b.user_id where a.is_deleted = 0 and b.role_id = "
                        + roleId;
            }
        } else {
            if (projectIds.size() > 0) {
                sql =
                    "select count(distinct a.id) from users a left join user_project b on a.id = b.user_id where a.is_deleted = 0 and b.project_id in (:projectIds) ";
            } else {
                sql = "select count(1) from users a where a.is_deleted = 0 ";
            }
        }
        sql = getQueryBuilder().buildRaw(sql, params, null, "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        if (projectIds.size() > 0) {
            paramMap.put("projectIds", projectIds);
        }
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 根据id查询用户
     *
     * @param id
     * @return
     */
    public User query(Long id) {
        String sql =
            "select u.*, t.account tenant_account, t.discount discount from users u left join tenant t on u.tenant_id = t.id where u.is_deleted = 0 and u.id = :id ";
        Map<String, Object> paramMap = MapTools.simpleMap("id", id);
        List<User> list = this.list(User.class, sql, paramMap);
        Assert.isTrue(list.size() <= 1, "more than one users in table");
        return list.isEmpty() ? null : list.get(0);
    }

    public List<User> listByIds(List<Long> ids) {
        String sql =
            "select u.*, t.account tenant_account, t.discount discount from users u left join tenant t on u.tenant_id = t.id where u.is_deleted = 0 and u.id in (:ids) ";
        Map<String, Object> paramMap = MapTools.simpleMap("ids", ids);
        return this.list(User.class, sql, paramMap);
    }

    /**
     * 根据邮箱查询用户
     *
     * @param email
     * @return
     */
    public User getByEmail(String email) {
        String sql = "select * from users where email = :email and is_deleted = 0";
        Map<String, Object> params = MapTools.simpleMap(Common.EMAIL, email);
        List<User> list = this.list(User.class, sql, params);
        Assert.isTrue(list.size() <= 1, "该邮箱已被注册");
        return list.isEmpty() ? null : list.get(0);
    }

    /**
     * 根据租户ID查询用户
     *
     * @param tenantId
     * @return
     */
    public List<User> listByTenant(Long tenantId) {
        String sql = "select a.* from users a where a.is_deleted = 0 and a.tenant_id = :tenantId";
        Map<String, Object> paramMap = MapTools.simpleMap("tenantId", tenantId);
        return this.list(User.class, sql, paramMap);
    }

    /**
     * 查询项目负责人
     *
     * @param projectId
     * @return
     */
    public List<User> getProjectManagers(Long projectId) {
        String sql =
            "select * from users u left join  user_project up on u.id = up.user_id where up.project_id = :projectId and up.project_admin = 1";
        Map<String, Object> params = MapTools.simpleMap("projectId", projectId);
        return this.list(User.class, sql, params);
    }

    public int projectUserCount(List<Param> params) {
        String sql =
            "select count(distinct b.id) from users b left join user_project up on b.id = up.user_id where b.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, params, null, "up");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.countQuery(sql, paramMap).intValue();
    }

    public List<User> listProjectUser(Integer page, Integer rows, List<Param> params, Map<String, String> sorter) {
        String sql = "select distinct a.id, a.account, a.email, a.phone, a.mobile, a.login_status, a.company, a.session_id, " +
                " a.tenant_id, a.depart_id, a.depart_ids, a.job_number, a.sex, a.is_manager, a.expired_date, a.origin," +
                " a.last_login_date, a.wechat, a.dingtalk, a.gmt_create, a.gmt_modify, a.creator_id, a.owner_id, a.mender_id, " +
                " a.name, a.status, a.is_deleted, a.props, a.remark, b.name tenant_name, b.id tenant_id, up.project_admin project_manager " +
                " from users a left join tenant b on b.id = a.tenant_id left join user_project up on a.id = up.user_id " +
                " where a.is_deleted = 0";
        sql = getQueryBuilder().buildRaw(sql, params, null, "up");
        sql = getQueryBuilder().buildRaw(sql, new Pager(page, rows, null, sorter), "a");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        return this.list(User.class, sql, paramMap);
    }

    public List<User> listManagerHasExpiredDate(String threeDaysAgo) {
        String sql = "select u.* from users u where is_deleted = 0 and status != 'EXPIRED' and status != 'ABNORMAL' and expired_date >= :threeDaysAgo";
        Map<String, Object> parms = MapTools.simpleMap("threeDaysAgo", threeDaysAgo);
        return this.list(User.class, sql,parms);
    }

    /**
     * 根据组织机构ID查询
     *
     * @param departId
     * @return
     */
    public List<User> listByDepartment(Long departId) {
        String sql = "select a.* from users a where a.is_deleted = 0 and a.depart_id = :departId";
        Map<String, Object> paramMap = MapTools.simpleMap("departId", departId);
        return this.list(User.class, sql, paramMap);
    }

    public List<User> listByOrigin(Long departId, String origin, boolean hasDept) {
        String sql = "select a.* from users a where a.is_deleted = 0 and a.origin = :origin";
        Map<String, Object> paramMap = MapTools.simpleMap("origin", origin);
        if (hasDept) {
            if (null != departId) {
                sql += " and a.depart_id = :departId";
                paramMap.put("departId", departId);
            }
        } else {
            sql += " and a.depart_id is null";
        }
        return this.list(User.class, sql, paramMap);
    }

    /**
     * @param departId
     * @param departIds
     * @return boolean
     * @Author: lxy
     * @Description: 根据租户id修改租户下所有用户组织
     * @Date 2021/6/15 14:36
     * @Param id
     **/
    public boolean updateDepartIdByTenantId(Long id, Long departId, String departIds) {
        String sql = "update users set depart_id = :departId , depart_ids = :departIds where tenant_id=:id";
        Map<String, Object> paramsMap = MapTools.simpleMap("id", id);
        paramsMap.put("departId", departId);
        paramsMap.put("departIds", departIds);
        return super.execute(sql, paramsMap) > 0;
    }

    /**
     * 根据用户组查询用户信息
     *
     * @param userGroupId
     * @return
     */
    public int countByGroupId(Long userGroupId) {
        String sql = "  select count(1) from users u,user_group_relation ugr"
            + "  where u.is_deleted = 0 and u.id = ugr.user_id and ugr.user_group_id = :userGroupId";
        Map<String, Object> params = MapTools.simpleMap("userGroupId", userGroupId);
        return this.countQuery(sql, params).intValue();
    }

    /**
     * 查询用户数量
     *
     * @param params
     * @param userGroupId
     * @return
     */
    public int countUser(List<Param> params, Long userGroupId) {
        String sql = "  select count(1) from users u inner join user_group_relation ugr on u.id = ugr.user_id"
            + "  where u.is_deleted = 0 and ugr.user_group_id = :userGroupId ";
        sql = getQueryBuilder().buildRaw(sql, params, null, "u");
        Map<String, Object> paramMap = getQueryBuilder().getParam(params);
        paramMap.put("userGroupId", userGroupId);
        return this.countQuery(sql, paramMap).intValue();
    }

    /**
     * 分页查询用户
     *
     * @param pager
     * @param userGroupId
     * @return
     */
    public List<User> listUser(Pager pager, Long userGroupId) {
        String sql = " select  u.* from users u inner join user_group_relation ugr on u.id = ugr.user_id "
            + " where u.is_deleted = 0 and ugr.user_group_id = :userGroupId ";
        sql = getQueryBuilder().buildRaw(sql, pager, "u");
        Map<String, Object> paramMap = getQueryBuilder().getParam(pager.getParams());
        paramMap.put("userGroupId", userGroupId);
        return this.list(User.class, sql, paramMap);
    }


    public User queryByUserId(String userId) {
        String sql = "select * from users where is_deleted = 0 and user_id = :userId";
        Map<String, Object> paramMap = MapTools.simpleMap("userId", userId);
        List<User> list = this.list(User.class, sql, paramMap);
        Assert.isTrue(list.size() <= 1, "more than one users in table");
        return list.isEmpty() ? null : list.get(0);
    }

}
